/*---------------------------------------------------------------
 * 					    pg_database.h
 *---------------------------------------------------------------
 *	Abstraction layer for libpq postgres database functions.
 *
 *  Copyright (C) 2004 Cerebral Tech, Inc.
 *--------------------------------------------------------------*/

#ifndef __PG_DATABASE_H
#define __PG_DATABASE_H

#include <libpq-fe.h>

static const char CONNECTION_STRING[] = "dbname=chinese";
static PGconn *con;
static PGresult *res;

static void db_disconnect()
{
	PQfinish(con);
}

bool db_connect()
{
	if(PQstatus(con) != CONNECTION_OK)
	{
		con = PQconnectdb(CONNECTION_STRING);
		if(PQstatus(con) != CONNECTION_OK) {
			fprintf(stderr, "Connection to database '%s' failed.\n", PQdb(con));
			fprintf(stderr, "%s", PQerrorMessage(con));
			db_disconnect();
			return false;
		} else {
			return true;
		}
	}
	return true;
}

PGresult *db_exec(const char *sql)
{
	if(!db_connect())
		return NULL;

	res = PQexec(con, sql);
	if (PQresultStatus(res) != PGRES_COMMAND_OK && PQresultStatus(res) != PGRES_TUPLES_OK)
    {
		fprintf(stderr, "DB OPERATION FAILED: %s", PQerrorMessage(con));
		PQclear(res);
		db_disconnect();
		return NULL;
	} else {
		return res;
	}
}

char *db_get_value(PGresult *res, int row, int field)
{
	return PQgetvalue(res, row, field);
}

int db_numrows(PGresult *res)
{
	return PQntuples(res);
}

int db_numfields(PGresult *res)
{
	return PQnfields(res);
}

void db_clear(PGresult *res)
{
	PQclear(res);
}

#endif

