/************************************************************************
 * Chinese Point-of-Sale system                                         *
 * Copyright (C) 2004  Cerebral Tech, Inc.                              *
 * Email: contact@cerebraltech.com                                      *
 * Web:   www.cerebraltech.com                                          *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 ***********************************************************************/

#include "pos.h"

// Global variables
CDKSCREEN *cdkscreen;
WINDOW *cursesWin;
CDKMENU *rolodexMenu;
CDKLABEL *date_label;
CDKLABEL *time_label;
CDKSCROLL *rightwindow;
pthread_t callerid_thread;
pthread_mutex_t mutex;
char callerid_phone_number[11];
char *orders[MAX_ORDER_SIZE];
char *specials[MAX_ORDER_SIZE];
char *specials_string[MAX_ORDER_SIZE];
double specialprices[MAX_ORDER_SIZE];
double prices[MAX_ORDER_SIZE];
int qty[MAX_ORDER_SIZE];
int nextorder, nextspecial;
char current_pickup_or_delivery;
char current_size;
int current_qty;
int customer_id;
bool customer_id_has_changed;
char remembered_callerid_phone_number[11];
bool go_directly_to_specials;

int main(int argc, char **argv)
{
	CDKLABEL *store_name_label;
	char *title[5];
	int ret;
	pthread_attr_t thread_attr;

	// initialize variables and stuff
	rightwindow = (CDKSCROLL*) NULL;
	strcpy(callerid_phone_number, "");
	nextorder = 0;
	nextspecial = 0;
	date_label = NULL;
	time_label = NULL;
	for(ret = 0; ret < 100; ret++)
		qty[ret] = 0;

	/* Set up CDK  */
	cursesWin = initscr();
	keypad(cursesWin, true);
	cdkscreen = initCDKScreen(cursesWin);

	/* Start CDK color. */
	initCDKColor();

	update_date_time();

	char store_name[100] = STORE_NAME_COLOR(STORE_NAME);
	title[0] = store_name;
	store_name_label = newCDKLabel(cdkscreen, CENTER, TOP, title, 1, FALSE, FALSE);

	// initialize caller ID thread
	pthread_mutex_init(&mutex, NULL);
	pthread_attr_init(&thread_attr);
	pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
	pthread_attr_setscope(&thread_attr, PTHREAD_SCOPE_SYSTEM);
	pthread_create(&callerid_thread, &thread_attr, callerid_func, NULL);

	// order screen
	char sql[500];
	struct timeval tv;
	fd_set fds;
	tv.tv_sec = 0;
	tv.tv_usec = 10000;
	while(true) { // eternally loop the main program
		refreshCDKScreen(cdkscreen);
		while(true) { // wait for interactive use or caller ID
			FD_ZERO(&fds);
			FD_SET(0, &fds); // stdin
			select(1, &fds, NULL, NULL, &tv);
			if(FD_ISSET(0, &fds)) {
				customer_id = 0;
				getch(); // throw away the character they pressed
				break;
			}

			pthread_mutex_lock(&mutex);
			if(strcmp(callerid_phone_number, "") != 0) {
				snprintf(sql, sizeof(sql)-1, "SELECT customer_id FROM Customer WHERE phone='%s' ORDER BY customer_id", callerid_phone_number);
				strcpy(remembered_callerid_phone_number, callerid_phone_number);
				strcpy(callerid_phone_number, "");
				pthread_mutex_unlock(&mutex);

				PGresult *result = db_exec(sql);

				if(db_numrows(result) > 0)
					customer_id = atoi(db_get_value(result, 0, 0));
				else
					customer_id = -1;

				db_clear(result);

				break;
			} else {
				pthread_mutex_unlock(&mutex);
			}
		}

		current_pickup_or_delivery = 'P';
		customer_id_has_changed = false;

		do {
			ret = orderScreen(&customer_id);
		} while(ret == 1);
	}

	/*
	create_menu();
	while(true) {
		// n00b around
		menu();
	}
	*/
}

void update_date_time()
{
	char *title[5];

	if(date_label != NULL) {
		destroyCDKLabel(date_label);
	}

	if(time_label != NULL) {
		destroyCDKLabel(time_label);
	}

	// n00b around with the time and date
	time_t current_time;
	struct tm *now;

	//char time_buffer[100];
	current_time = time(NULL);
	now = localtime(&current_time);
	
	char date[50];
	snprintf(date, sizeof(date)-1, DATE_COLOR("%d/%02d/%02d"),
			now->tm_mon+1,
			now->tm_mday,
			now->tm_year+1900);
	title[0]= date;
	date_label = newCDKLabel(cdkscreen, LEFT, TOP, title, 1, FALSE, FALSE);

	char time_buf[50];
	snprintf(time_buf, sizeof(time_buf)-1, TIME_COLOR("%d:%02d %s"),
			(now->tm_hour > 12 ? now->tm_hour - 12 : (now->tm_hour == 0 ? 12 : now->tm_hour)),
			now->tm_min,
			(now->tm_hour < 12 ? "AM" : "PM"));
  
	title[0] = time_buf;
	time_label = newCDKLabel(cdkscreen, RIGHT, TOP, title, 1, FALSE, FALSE);

	//refreshCDKScreen(cdkscreen);
	drawCDKLabel(time_label, FALSE);
	drawCDKLabel(date_label, FALSE);
}

void create_menu()
{
	char *menulist[MAX_MENU_ITEMS][MAX_SUB_ITEMS];
	int subMenuSize[10], menuLocations[10];

	/* Create the menu lists. */
	menulist[0][0] = "</B/5>Administration      ";
	menulist[0][1] = "</B/5>New Customer        ";
	menulist[0][2] = "</B/5>View Customers      ";
	menulist[0][3] = "</B/5>New Menu Item       ";
	menulist[0][4] = "</B/5>View Menu Items     ";
	menulist[0][5] = "</B/5>New Special Field   ";
	menulist[0][6] = "</B/5>View Special Fields ";

	menulist[1][0] = "</B/5>Help                ";
	menulist[1][1] = "</B/5>About This Software ";

	menulist[2][0] = "</B/5>QUIT                ";
	menulist[2][1] = "</B/5>Leave Back Office   ";
	menulist[2][2] = "</B/5>Shut Down System    ";

	/* Set up the sub-menu sizes and their locations. */
	subMenuSize[0] = 7; menuLocations[0] = LEFT;
	subMenuSize[1] = 2; menuLocations[1] = LEFT;
	subMenuSize[2] = 3; menuLocations[2] = RIGHT;

	/* Create the menu. */
	rolodexMenu = newCDKMenu(cdkscreen, menulist, 3,
				  subMenuSize, menuLocations,
				  TOP, A_BOLD, A_REVERSE);
}

int menu()
{
	int selection, ret;

	/* Draw the CDK screen. */
	refreshCDKScreen(cdkscreen);

	//int clock_count = 0;
	for(;;)
	{
		/*
		++clock_count;
		if(clock_count > 10000000)
		{
			current_time = time(NULL);
			now = localtime(&current_time);

			snprintf(time_buf, sizeof(time_buf)-1, "<R>%02d:%02d:%02d",
					now->tm_hour,
					now->tm_min,
					now->tm_sec);

			title[0] = time_buf;
			setCDKLabel(time_label, title, 1, FALSE);
			drawCDKLabel(time_label, FALSE);
			clock_count = 0;
		}
		*/

		selection= activateCDKMenu(rolodexMenu, (chtype *)NULL);

		if(selection == 0) // new customer
		{
			ret = editCustomer(false, -1);
		}
		else if(selection == 1) // view customers
		{
			ret = viewCustomers(true);
		}
		else if(selection == 2) // new menu item
		{
			ret = editMenuItem(false, "none");
		}
		else if(selection == 3) // view menu items
		{
			ret = viewMenuItems();
		}
		else if(selection == 4) // new special
		{
			ret = editSpecial(false, -1);
		}
		else if(selection == 5) // view specials
		{
			ret = viewSpecials();
		}
		else if(selection == 100) // help/about
		{
			notImplemented();
		}
		else if(selection == 200) // leave back office
		{
			return 0;
			/*
			customer_id = -1;
			do {
				ret = orderScreen(&customer_id);
			} while(ret == 1);
			*/
		}
		else if(selection == 201) // quit entirely
		{
			ret = confirm("ARE YOU SURE YOU WANT TO QUIT?");
			if(ret == 0) // quitting
			{
				destroyCDKMenu(rolodexMenu);
				//destroyCDKLabel(time_label);
				//destroyCDKLabel(date_label);
				//destroyCDKLabel(store_name_label);
				destroyCDKScreen(cdkscreen);
				delwin(cursesWin);
	
				endCDK();
				pthread_kill(callerid_thread, SIGKILL);
				exit(0);
			}
		}

		refreshCDKScreen(cdkscreen);
		//setCDKLabel(rolodexTitle, time_buffer, 1, FALSE);
		//drawCDKLabel(rolodexTitle, rolodexTitle->box);
		//sleep(1);
	}
}

int confirm(char *message)
{
	char *buttons[2], *mesg[15];
	char temp[500];

	snprintf(temp, sizeof(temp), "<C>%s", message);
	mesg[0] = copyChar(temp);
	buttons[0] = "</B/24><YES>";
	buttons[1] = "</B/16><NO>";

	return popupDialog(cdkscreen, mesg, 1, buttons, 2);
}

char *pad(char *string, int length)
{
	int i=0;
	int string_length = strlen(string);

	if(length < string_length && length > 3)
	{
		string[length-3] = '.';
		string[length-2] = '.';
		string[length-1] = '.';
		string[length] = '\0';
		return string;
	} 

	char *new_string = (char *)malloc(length + 1);
	strncpy(new_string, string, length+1);
	
	for(i = string_length; i < length; i++)
	{
		new_string[i] = ' ';
	}
	new_string[i] = '\0';
	return new_string;
}

char *addslashes(char *string)
{
	char newstring[500];

	int i, j;
	for(i = 0, j = 0; i < strlen(string); i++, j++) {
		if(string[i] == '\'') {
			newstring[j] = '\\';
			j++;
			newstring[j] = '\'';
		} else {
			newstring[j] = string[i];
		}
	}
	newstring[j] = '\0';

	strcpy(string, newstring);
	return string;
}

int viewMenuItems()
{
	/* Set up the data needed for the scrolling list. */
	CDKSCROLL *nameList = (CDKSCROLL *)NULL;
	char *list[MAX_ITEMS];
	char temp[500];
	int ret;

	db_connect();
	char sql[500];
	snprintf(sql, sizeof(sql)-1, "SELECT item_id, name_eng, name_chi, price FROM Item ORDER BY item_id");
	PGresult *result = db_exec(sql);

	int num_rows = db_numrows(result);
	int i;
	for(i = 0; i < num_rows; i++)
	{
		snprintf(temp, sizeof(temp)-1, "</B/29>%s    %s    %s    $%0.2f",
				pad(db_get_value(result, i, 0), 5),
				pad(db_get_value(result, i, 1), 30),
				pad(db_get_value(result, i, 2), 25),
				atof(db_get_value(result, i, 3))
				);
		list[i] = copyChar(temp);
	}

	snprintf(temp, sizeof(temp)-1, "<C></24></B>Menu Items in Database\n<C>Choose an Item to Edit, or ESCAPE to go back.\n");

	/* Create the scrolling list. */
	nameList = newCDKScroll(cdkscreen, CENTER, CENTER, RIGHT,
							-6, -4, temp, list,
							num_rows,
							NUMBERS, A_REVERSE, TRUE, FALSE);

	/* Clean up. */
	for(i = 0; i < num_rows; i++)
	{
		freeChar(list[i]);
	}

	int selection = 0;
	// Get the information they want to view.      
	selection = activateCDKScroll(nameList, (chtype *)NULL);
	if(nameList->exitType == vNORMAL) {
		/* if they select a record take them to the edit screen */
		eraseCDKScroll(nameList);
		destroyCDKScroll(nameList);
		ret = editMenuItem(true, db_get_value(result, selection, 0));
		db_clear(result);
		//db_disconnect();
		return 0;
	} else if(nameList->exitType == vESCAPE_HIT) {
		eraseCDKScroll(nameList);
		destroyCDKScroll(nameList);
		db_clear(result);
		//db_disconnect();
		return 0;
	}

	// should never reach here, unless something truly horrible happens
	return 0;
}

int editMenuItem(bool edit_flag, char *db_item_id)
{
	/* Declare local variables. */
	CDKENTRY *item_id_entry, *name_eng_entry, *name_chi_entry;
	CDKENTRY *price_entry;
	CDKLABEL *title_label;

	char *item_id;
	char *name_eng;
	char *name_chi;
	char price_temp[256];
	double price;
	char *price_ptr;

	//CDKTEMPLATE *phoneTemplate;
	char *buttons[5], *mesg[15];
	int ret;

	PGresult *result, *result2;

	char *title[1];
	title[0] = "<C></24></B>Add/Edit a Menu Item";
	title[1] = "<C></B>TAB advances, ESCAPE quits.\n";
	title_label = newCDKLabel(cdkscreen, CENTER, TOP, title, 2, FALSE, FALSE);

	drawCDKLabel(title_label, false);
  
	item_id_entry = newCDKEntry(cdkscreen, CENTER, 5,
								NULL, "</B/5>ID:              ",
								A_NORMAL,
								'_', vUMIXED, 40, 1, 256,
								TRUE, FALSE);
	name_eng_entry = newCDKEntry(cdkscreen, CENTER, 8,
								NULL, "</B/5>Name (English):  ",
								A_NORMAL,
								'_', vCHAR, 40, 0, 256,
								TRUE, FALSE);
	name_chi_entry = newCDKEntry(cdkscreen, CENTER, 11,
								NULL, "</B/5>Name (Chinese):  ",
								A_NORMAL,
								'_', vMIXED, 40, 0, 256,
								TRUE, FALSE);
	price_entry = newCDKEntry(cdkscreen, CENTER, 14,
								NULL, "</B/5>Price:           ",
								A_NORMAL,
								'_', vUMIXED, 40, 0, 256,
								TRUE, FALSE);

	if(edit_flag)
	{
		db_connect();
		char sql[500];
		snprintf(sql, sizeof(sql)-1, "SELECT item_id, name_eng, name_chi, price FROM Item WHERE item_id='%s'", db_item_id);
		result = db_exec(sql);

		setCDKEntryValue(item_id_entry, db_get_value(result, 0, 0));
		setCDKEntryValue(name_eng_entry, db_get_value(result, 0, 1));
		setCDKEntryValue(name_chi_entry, db_get_value(result, 0, 2));
		snprintf(price_temp, sizeof(price_temp)-1, "%s",  db_get_value(result, 0, 3));
		setCDKEntryValue(price_entry,  price_temp);

		db_clear(result);
		db_disconnect();
	}

	/* Get the information. */
	for(;;)
	{
		/* Draw the widgets on the screen. */
		drawCDKEntry(item_id_entry, item_id_entry->box);
		drawCDKEntry(name_eng_entry, name_eng_entry->box);
		drawCDKEntry(name_chi_entry, name_chi_entry->box);
		drawCDKEntry(price_entry, price_entry->box);

		/* Check the response of the popup dialog box. */
		/*
		if(price_entry->exitType == vESCAPE_HIT)
		{
			freeChar(item_id);
			freeChar(name_eng);
			freeChar(name_chi);

			destroyCDKEntry(item_id_entry);
			destroyCDKEntry(name_eng_entry);
			destroyCDKEntry(name_chi_entry);
			destroyCDKEntry(price_entry);
			destroyCDKLabel(title_label);
			return 0;
		}

		*/

		/* Activate the entries to get the information. */
	  
		item_id = copyChar(activateCDKEntry(item_id_entry, NULL));
		if(item_id == NULL){ break; }
		addslashes(item_id);
		  
		name_eng = copyChar(activateCDKEntry(name_eng_entry, NULL));
		if(name_eng == NULL){ break; }
		addslashes(name_eng);

		name_chi = copyChar(activateCDKEntry(name_chi_entry, NULL));
		if(name_chi == NULL){ break; }
		addslashes(name_chi);
	 
		price_ptr = copyChar(activateCDKEntry(price_entry, NULL));
		if(price_ptr == NULL){ break; }
		price = atof(price_ptr);

		// ask about specials
		db_connect();
		char sql[500];
		snprintf(sql, sizeof(sql)-1, "SELECT special_field_id, name_eng, name_chi, price FROM special_field ORDER BY special_field_id");
		result = db_exec(sql);

		int num_rows = db_numrows(result);
		if(num_rows > 0) {
			ret = confirm("Do you want to associate any Special Fields with this Menu Item?");
			if(ret == 0) {
				CDKSCROLL *nameList = (CDKSCROLL *)NULL;
				char *list[MAX_ITEMS], temp[500];
				int i;

				// loop through and let the user select as many specials as they want
				while(true) {
					for(i = 0; i < num_rows; i++)
					{
						snprintf(sql, sizeof(sql), "SELECT * FROM item_special_field WHERE item_id='%s' AND special_field_id='%s'", (strcmp(db_item_id, "none") == 0 ? item_id : db_item_id), db_get_value(result, i, 0));
						result2 = db_exec(sql);

						snprintf(temp, sizeof(temp)-1, "</B/29>%c %s  %s  $%0.2f",
								(db_numrows(result2) == 1 ? '*' : ' '),
								pad(db_get_value(result, i, 1), 15),
								pad(db_get_value(result, i, 2), 15),
								atof(db_get_value(result, i, 3))
								);
						list[i] = copyChar(temp);

						db_clear(result2);
					}

					snprintf(temp, sizeof(temp)-1, "<C></24></B>Special Fields in Database\n<C>Choose Special Fields, or ESCAPE to go back.\n");

					/* Create the scrolling list. */
					nameList = newCDKScroll(cdkscreen, CENTER, CENTER, RIGHT,
											-6, -4, temp, list, num_rows,
											NUMBERS, A_REVERSE, TRUE, FALSE);

					/* Clean up. */
					for(i = 0; i < num_rows; i++)
					{
						freeChar(list[i]);
					}

					int selection = 0;
					// Get the information they want to view.      
					selection = activateCDKScroll(nameList, (chtype *)NULL);
					if(nameList->exitType == vNORMAL) {
						snprintf(sql, sizeof(sql), "SELECT * FROM item_special_field WHERE item_id='%s' AND special_field_id='%s'", (strcmp(db_item_id, "none") == 0 ? item_id : db_item_id), db_get_value(result, selection, 0));
						result2 = db_exec(sql);
						if(db_numrows(result2) == 1) {
							db_clear(result2);
							snprintf(sql, sizeof(sql), "DELETE FROM item_special_field WHERE item_id='%s' AND special_field_id='%s'", (strcmp(db_item_id, "none") == 0 ? item_id : db_item_id), db_get_value(result, selection, 0));
							db_exec(sql);
						} else {
							db_clear(result2);
							snprintf(sql, sizeof(sql), "INSERT INTO item_special_field (item_id, special_field_id) VALUES ('%s','%s')", (strcmp(db_item_id, "none") == 0 ? item_id : db_item_id), db_get_value(result, selection, 0));
							db_exec(sql);
						}

						eraseCDKScroll(nameList);
						destroyCDKScroll(nameList);
						//db_disconnect();
						refreshCDKScreen(cdkscreen);
					} else if(nameList->exitType == vESCAPE_HIT) {
						//nextspecial = 0;

						eraseCDKScroll(nameList);
						destroyCDKScroll(nameList);
						//db_disconnect();
						refreshCDKScreen(cdkscreen);
						break;
					}
				}
			}
		}
	  
		/* Determine if the user wants to submit the info. */
		mesg[0] = "<C></U>Confirm Entry";
		mesg[1] = "<C>Do you want to add this information?";
		buttons[0] = "</B/24><Add Menu Item>";
		buttons[1] = "</B/16><Cancel>";
		buttons[2] = "</B/32><Modify Information>";
		ret = popupDialog(cdkscreen, mesg, 2, buttons, 3);

		if(ret == 0)
		{
			/* The user wants to submit the information. */
			destroyCDKEntry(item_id_entry);
			destroyCDKEntry(name_eng_entry);
			destroyCDKEntry(name_chi_entry);
			destroyCDKEntry(price_entry);
			destroyCDKLabel(title_label);

			db_connect();
			if(edit_flag)
			{
				char sql[500];
				snprintf(sql, sizeof(sql)-1, "UPDATE item SET item_id='%s', name_eng='%s', name_chi='%s', price='%f' WHERE item_id='%s'", item_id, name_eng, name_chi, price, db_item_id);
				db_exec(sql);

				snprintf(sql, sizeof(sql)-1, "UPDATE item_special_field SET item_id='%s' WHERE item_id='%s'", item_id, db_item_id);
				db_exec(sql);
			} else {
				char sql[500];
				snprintf(sql, sizeof(sql)-1, "INSERT INTO item (item_id, name_eng, name_chi, price) VALUES ('%s', '%s', '%s', '%f')", item_id, name_eng, name_chi, price);
				db_exec(sql);
			}

			db_disconnect();
			return 0;
		}
		else if(ret == 1)
		{
			/* The user does not want to submit the information. */
			break;
		}
		else
		{
			/* The user wants to edit the information again. */
			freeChar(item_id);
			freeChar(name_eng);
			freeChar(name_chi);
		}
	}

	// cleanup
	/*freeChar(item_id);
	freeChar(name_eng);
	freeChar(name_chi);
	freeChar(price_ptr);
	*/

	destroyCDKEntry(item_id_entry);
	destroyCDKEntry(name_eng_entry);
	destroyCDKEntry(name_chi_entry);
	destroyCDKEntry(price_entry);
	destroyCDKLabel(title_label);
	return 0;
}

int viewSpecials()
{
	/* Set up the data needed for the scrolling list. */
	CDKSCROLL *nameList = (CDKSCROLL *)NULL;
	char *list[MAX_ITEMS];
	char temp[500];
	int ret;

	db_connect();
	char sql[500];
	snprintf(sql, sizeof(sql)-1, "SELECT special_field_id, name_eng, name_chi, price FROM special_field ORDER BY special_field_id");
	PGresult *result = db_exec(sql);

	int num_rows = db_numrows(result);
	int i;
	for(i = 0; i < num_rows; i++)
	{
		snprintf(temp, sizeof(temp)-1, "</B/29>%s    %s    %s    $%0.2f",
				pad(db_get_value(result, i, 0), 5),
				pad(db_get_value(result, i, 1), 30),
				pad(db_get_value(result, i, 2), 25),
				atof(db_get_value(result, i, 3))
				);
		list[i] = copyChar(temp);
	}

	snprintf(temp, sizeof(temp)-1, "<C></24></B>Special Fields in Database\n<C>Choose a Special to Edit, or ESCAPE to go back.\n");

	/* Create the scrolling list. */
	nameList = newCDKScroll(cdkscreen, CENTER, CENTER, RIGHT,
							-6, -4, temp, list,
							num_rows,
							NUMBERS, A_REVERSE, TRUE, FALSE);

	/* Clean up. */
	for(i = 0; i < num_rows; i++)
	{
		freeChar(list[i]);
	}

	int selection = 0;
	// Get the information they want to view.      
	selection = activateCDKScroll(nameList, (chtype *)NULL);
	if(nameList->exitType == vNORMAL) {
		/* if they select a record take them to the edit screen */
		eraseCDKScroll(nameList);
		destroyCDKScroll(nameList);
		ret = editSpecial(true, atoi(db_get_value(result, selection, 0)));
		db_clear(result);
		//db_disconnect();
		return 0;
	} else if(nameList->exitType == vESCAPE_HIT) {
		eraseCDKScroll(nameList);
		destroyCDKScroll(nameList);
		db_clear(result);
		//db_disconnect();
		return 0;
	}

	// should never reach here, unless something truly horrible happens
	return 0;
}

int editSpecial(bool edit_flag, int db_special_id)
{
	/* Declare local variables. */
	CDKENTRY *special_field_id_entry, *name_eng_entry, *name_chi_entry;
	CDKENTRY *price_entry;
	CDKLABEL *title_label;

	int special_field_id;
	char *special_field_ptr;
	char *name_eng;
	char *name_chi;
	char price_temp[256];
	double price;
	char *price_ptr;

	//CDKTEMPLATE *phoneTemplate;
	char *buttons[5], *mesg[15];
	int ret;

	/* Define the widgets. */

	char *title[1];
	title[0] = "<C></24></B>Add/Edit a Special Field";
	title[1] = "<C></B>TAB advances, ESCAPE quits.\n";
	title_label = newCDKLabel(cdkscreen, CENTER, TOP, title, 2, FALSE, FALSE);

	drawCDKLabel(title_label, false);
  
	special_field_id_entry = newCDKEntry(cdkscreen, CENTER, 5,
								NULL, "</B/5>ID:              ",
								A_NORMAL,
								'_', vUMIXED, 40, 1, 256,
								TRUE, FALSE);
	name_eng_entry = newCDKEntry(cdkscreen, CENTER, 8,
								NULL, "</B/5>Name (English):  ",
								A_NORMAL,
								'_', vCHAR, 40, 0, 256,
								TRUE, FALSE);
	name_chi_entry = newCDKEntry(cdkscreen, CENTER, 11,
								NULL, "</B/5>Name (Chinese):  ",
								A_NORMAL,
								'_', vMIXED, 40, 0, 256,
								TRUE, FALSE);
	price_entry = newCDKEntry(cdkscreen, CENTER, 14,
								NULL, "</B/5>Price:           ",
								A_NORMAL,
								'_', vUMIXED, 40, 0, 256,
								TRUE, FALSE);

	if(edit_flag)
	{
		db_connect();
		char sql[500];
		snprintf(sql, sizeof(sql)-1, "SELECT special_field_id, name_eng, name_chi, price FROM special_field WHERE special_field_id='%d'", db_special_id);
		PGresult *result = db_exec(sql);

		setCDKEntryValue(special_field_id_entry, db_get_value(result, 0, 0));
		setCDKEntryValue(name_eng_entry, db_get_value(result, 0, 1));
		setCDKEntryValue(name_chi_entry, db_get_value(result, 0, 2));
		snprintf(price_temp, sizeof(price_temp)-1, "%s",  db_get_value(result, 0, 3));
		setCDKEntryValue(price_entry, price_temp);

		db_clear(result);
		db_disconnect();
	}

	/* Get the information. */
	for(;;)
	{
		/* Draw the widgets on the screen. */
		drawCDKEntry(special_field_id_entry, special_field_id_entry->box);
		drawCDKEntry(name_eng_entry, name_eng_entry->box);
		drawCDKEntry(name_chi_entry, name_chi_entry->box);
		drawCDKEntry(price_entry, price_entry->box);

		/* Check the response of the popup dialog box. */
		/*
		if(price_entry->exitType == vESCAPE_HIT)
		{
			freeChar(item_id);
			freeChar(name_eng);
			freeChar(name_chi);

			destroyCDKEntry(item_id_entry);
			destroyCDKEntry(name_eng_entry);
			destroyCDKEntry(name_chi_entry);
			destroyCDKEntry(price_entry);
			destroyCDKLabel(title_label);
			return 0;
		}

		*/

		/* Activate the entries to get the information. */
	  
		special_field_ptr = copyChar(activateCDKEntry(special_field_id_entry, NULL));
		if(special_field_ptr == NULL){ break; }
		special_field_id = atoi(special_field_ptr);
		  
		name_eng = copyChar(activateCDKEntry(name_eng_entry, NULL));
		if(name_eng == NULL){ break; }
		addslashes(name_eng);

		name_chi = copyChar(activateCDKEntry(name_chi_entry, NULL));
		if(name_chi == NULL){ break; }
		addslashes(name_chi);
	 
		price_ptr = copyChar(activateCDKEntry(price_entry, NULL));
		if(price_ptr == NULL){ break; }
		price = atof(price_ptr);
	  
		/* Determine if the user wants to submit the info. */
		mesg[0] = "<C></U>Confirm Entry";
		mesg[1] = "<C>Do you want to add this information?";
		buttons[0] = "</B/24><Add Special Field>";
		buttons[1] = "</B/16><Cancel>";
		buttons[2] = "</B/32><Modify Information>";
		ret = popupDialog(cdkscreen, mesg, 2, buttons, 3);

		if(ret == 0)
		{
			/* The user wants to submit the information. */
			destroyCDKEntry(special_field_id_entry);
			destroyCDKEntry(name_eng_entry);
			destroyCDKEntry(name_chi_entry);
			destroyCDKEntry(price_entry);
			destroyCDKLabel(title_label);

			db_connect();
			if(edit_flag)
			{
				char sql[500];
				snprintf(sql, sizeof(sql)-1, "UPDATE special_field SET special_field_id='%d', name_eng='%s', name_chi='%s', price='%f' WHERE special_field_id='%d'", special_field_id, name_eng, name_chi, price, db_special_id);
				db_exec(sql);
			} else {
				char sql[500];
				snprintf(sql, sizeof(sql)-1, "INSERT INTO special_field (special_field_id, name_eng, name_chi, price) VALUES ('%d', '%s', '%s', '%f')", special_field_id, name_eng, name_chi, price);
				db_exec(sql);
			}

			db_disconnect();
			return 0;
		}
		else if(ret == 1)
		{
			/* The user does not want to submit the information. */
			break;
		}
		else
		{
			/* The user wants to edit the information again. */
			freeChar(special_field_ptr);
			freeChar(name_eng);
			freeChar(name_chi);
		}
	}

	// cleanup
	/*freeChar(item_id);
	freeChar(name_eng);
	freeChar(name_chi);
	freeChar(price_ptr);
	*/

	destroyCDKEntry(special_field_id_entry);
	destroyCDKEntry(name_eng_entry);
	destroyCDKEntry(name_chi_entry);
	destroyCDKEntry(price_entry);
	destroyCDKLabel(title_label);
	return 0;
}

int viewCustomers(bool edit_flag)
{
	/* Set up the data needed for the scrolling list. */
	CDKSCROLL *nameList = (CDKSCROLL *)NULL;
	char *list[MAX_ITEMS];
	char temp[500];
	int ret;

	db_connect();
	char sql[500];
	snprintf(sql, sizeof(sql)-1, "SELECT customer_id, name, street, city, state, zip, phone, number_calls, other FROM Customer ORDER BY customer_id");
	PGresult *result = db_exec(sql);

	int num_rows = db_numrows(result);
	int i;
	for(i = 0; i < num_rows; i++)
	{
		/*
		snprintf(temp, sizeof(temp)-1, "</B/29>%s    %s    %s, %s    %s    %s    %s    %s",
				pad(db_get_value(result, i, 1), 30),
				pad(db_get_value(result, i, 2), 20),
				pad(db_get_value(result, i, 3), 10),
				pad(db_get_value(result, i, 4), 2),
				pad(db_get_value(result, i, 5), 6),
				pad(db_get_value(result, i, 6), 12),
				pad(db_get_value(result, i, 7), 10),
				pad(db_get_value(result, i, 8), 20)
				);
		*/
		snprintf(temp, sizeof(temp)-1, "</B/29>%s    %s",
				pad(db_get_value(result, i, 1), 40),
				pad(db_get_value(result, i, 6), 12)
				);

		list[i] = copyChar(temp);
	}

	if(edit_flag)
		snprintf(temp, sizeof(temp)-1, "<C></24></B>Customers in Database\n<C>Choose a Customer to Edit, or ESCAPE to go back.\n");
	else
		snprintf(temp, sizeof(temp)-1, "<C></24></B>Customers in Database\n<C>Choose a Customer, or ESCAPE to go back.\n");

	/* Create the scrolling list. */
	nameList = newCDKScroll(cdkscreen, CENTER, CENTER, RIGHT,
							-6, -4, temp, list,
							num_rows,
							NUMBERS, A_REVERSE, TRUE, FALSE);

	/* Clean up. */
	for(i = 0; i < num_rows; i++)
	{
		freeChar(list[i]);
	}

	int selection = 0;
	// Get the information they want to view.      
	selection = activateCDKScroll(nameList, (chtype *)NULL);
	if(nameList->exitType == vNORMAL) {
		/* if they select a record take them to the edit screen */
		eraseCDKScroll(nameList);
		destroyCDKScroll(nameList);

		if(edit_flag)
			ret = editCustomer(true, atoi(db_get_value(result, selection, 0)));
		else
			ret = atoi(db_get_value(result, selection, 0));

		db_clear(result);
		//db_disconnect();
		return ret;
	} else if(nameList->exitType == vESCAPE_HIT) {
		eraseCDKScroll(nameList);
		destroyCDKScroll(nameList);
		db_clear(result);
		//db_disconnect();
		return 0;
	}

	// should never reach here, unless something truly horrible happens
	return 0;
}

int editCustomer(bool edit_flag, int db_customer_id)
{
	/* Declare local variables. */
	CDKENTRY *name_entry, *street_entry;
	CDKENTRY *city_entry, *state_entry, *zip_entry, *phone_entry;
	CDKENTRY *number_calls_entry, *other_entry;
	CDKLABEL *title_label;

	char *name;
	char *street;
	char *city;
	char *state;
	char *zip;
	char *phone;
	char *number_calls;
	char *other;

	char *buttons[5], *mesg[15];
	int ret;

	char *title[1];
	title[0] = "<C></24></B>Add/Edit a Customer Item";
	title[1] = "<C></B>TAB advances, ESCAPE quits.\n";
	title_label = newCDKLabel(cdkscreen, CENTER, TOP, title, 2, FALSE, FALSE);

	drawCDKLabel(title_label, false);
  
	name_entry = newCDKEntry(cdkscreen, 10, 5,
								NULL, "</B/5>Name:         ",
								A_NORMAL,
								'_', vCHAR, 40, 0, 256,
								TRUE, FALSE);
	street_entry = newCDKEntry(cdkscreen, 10, 8,
								NULL, "</B/5>Address:      ",
								A_NORMAL,
								'_', vMIXED, 40, 0, 256,
								TRUE, FALSE);
	city_entry = newCDKEntry(cdkscreen, 10, 11,
								NULL, "</B/5>City:         ",
								A_NORMAL,
								'_', vMIXED, 40, 0, 256,
								TRUE, FALSE);
	state_entry = newCDKEntry(cdkscreen, 10, 14,
								NULL, "</B/5>State:        ",
								A_NORMAL,
								'_', vCHAR, 10, 0, 2,
								TRUE, FALSE);
	zip_entry = newCDKEntry(cdkscreen, 40, 14,
								NULL, "</B/5>Zip Code: ",
								A_NORMAL,
								'_', vINT, 14, 0, 5,
								TRUE, FALSE);
	phone_entry = newCDKEntry(cdkscreen, 10, 17,
								NULL, "</B/5>Phone:        ",
								A_NORMAL,
								'_', vMIXED, 40, 0, 256,
								TRUE, FALSE);
	number_calls_entry = newCDKEntry(cdkscreen, 10, 20,
								NULL, "</B/5>Number Calls: ",
								A_NORMAL,
								'_', vMIXED, 40, 0, 256,
								TRUE, FALSE);
	other_entry = newCDKEntry(cdkscreen, 10, 23,
								NULL, "</B/5>Other:        ",
								A_NORMAL,
								'_', vMIXED, 40, 0, 256,
								TRUE, FALSE);

	if(edit_flag)
	{
		db_connect();
		char sql[500];
		snprintf(sql, sizeof(sql)-1, "SELECT name, street, city, state, zip, phone, number_calls, other FROM Customer WHERE customer_id='%d'", db_customer_id);
		PGresult *result = db_exec(sql);

		setCDKEntryValue(name_entry, db_get_value(result, 0, 0));
		setCDKEntryValue(street_entry, db_get_value(result, 0, 1));
		setCDKEntryValue(city_entry, db_get_value(result, 0, 2));
		setCDKEntryValue(state_entry, db_get_value(result, 0, 3));
		setCDKEntryValue(zip_entry, db_get_value(result, 0, 4));
		setCDKEntryValue(phone_entry, db_get_value(result, 0, 5));
		setCDKEntryValue(number_calls_entry, db_get_value(result, 0, 6));
		setCDKEntryValue(other_entry, db_get_value(result, 0, 7));

		db_clear(result);
		db_disconnect();
	}

	/* Get the information. */
	for(;;)
	{
		/* Draw the widgets on the screen. */
		drawCDKEntry(name_entry, name_entry->box);
		drawCDKEntry(street_entry, street_entry->box);
		drawCDKEntry(city_entry, city_entry->box);
		drawCDKEntry(state_entry, state_entry->box);
		drawCDKEntry(zip_entry, zip_entry->box);
		drawCDKEntry(phone_entry, phone_entry->box);
		drawCDKEntry(number_calls_entry, number_calls_entry->box);
		drawCDKEntry(other_entry, other_entry->box);

		/* Activate the entries to get the information. */
	  
		name = copyChar(activateCDKEntry(name_entry, NULL));
		if(name == NULL){ break; }
		addslashes(name);

		street = copyChar(activateCDKEntry(street_entry, NULL));
		if(street == NULL){ break; }
		addslashes(street);
	 
		city = copyChar(activateCDKEntry(city_entry, NULL));
		if(city == NULL){ break; }
		addslashes(city);
	  
		state = copyChar(activateCDKEntry(state_entry, NULL));
		if(state == NULL){ break; }
		addslashes(state);
	  
		zip = copyChar(activateCDKEntry(zip_entry, NULL));
		if(zip == NULL){ break; }
		addslashes(zip);
	  
		phone = copyChar(activateCDKEntry(phone_entry, NULL));
		if(phone == NULL){ break; }
		addslashes(phone);
	  
		number_calls = copyChar(activateCDKEntry(number_calls_entry, NULL));
		if(number_calls == NULL){ break; }
		addslashes(number_calls);
	  
		other = copyChar(activateCDKEntry(other_entry, NULL));
		if(other == NULL){ break; }
		addslashes(other);
	  
		/* Determine if the user wants to submit the info. */
		mesg[0] = "<C></U>Confirm Entry";
		mesg[1] = "<C>Do you want to add this information?";

		if(edit_flag)
			buttons[0] = "</B/24><Update Customer>";
		else
			buttons[0] = "</B/24><Add Customer>";

		buttons[1] = "</B/16><Cancel>";
		buttons[2] = "</B/32><Modify Information>";
		ret = popupDialog(cdkscreen, mesg, 2, buttons, 3);

		if(ret == 0)
		{
			/* The user wants to submit the information. */
			destroyCDKEntry(name_entry);
			destroyCDKEntry(street_entry);
			destroyCDKEntry(city_entry);
			destroyCDKEntry(state_entry);
			destroyCDKEntry(zip_entry);
			destroyCDKEntry(phone_entry);
			destroyCDKEntry(number_calls_entry);
			destroyCDKEntry(other_entry);
			destroyCDKLabel(title_label);

			db_connect();
			if(edit_flag)
			{
				char sql[500];
				snprintf(sql, sizeof(sql)-1, "UPDATE customer SET name='%s', street='%s', city='%s', state='%s', zip='%s', phone='%s', number_calls='%s', other='%s' WHERE customer_id='%d'", name, street, city, state, zip, phone, number_calls, other, db_customer_id);
				db_exec(sql);
			} else {
				char sql[500];
				snprintf(sql, sizeof(sql)-1, "INSERT INTO customer (name, street, city, state, zip, phone, number_calls, other) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')", name, street, city, state, zip, phone, number_calls, other);
				db_exec(sql);
			}
			db_disconnect();
			return 0;
		}
		else if(ret == 1)
		{
			/* The user does not want to submit the information. */
			break;
		}
		else
		{
			/* The user wants to edit the information again. */
			freeChar(name);
			freeChar(street);
			freeChar(city);
			freeChar(state);
			freeChar(zip);
			freeChar(phone);
			freeChar(number_calls);
			freeChar(other);
		}
	}

	destroyCDKEntry(name_entry);
	destroyCDKEntry(street_entry);
	destroyCDKEntry(city_entry);
	destroyCDKEntry(state_entry);
	destroyCDKEntry(zip_entry);
	destroyCDKEntry(phone_entry);
	destroyCDKEntry(number_calls_entry);
	destroyCDKEntry(other_entry);
	destroyCDKLabel(title_label);
	return 0;
}

int orderScreen(int *customer_id)
{
	/* Set up the data needed for the scrolling list. */
	CDKSCROLL *leftwindow = (CDKSCROLL*)NULL;
	CDKALPHALIST *nameList = (CDKALPHALIST *)NULL;
	char *list[MAX_ITEMS];
	char temp[500], temp2[500];
	int ret, num_rows, i, ndx;

	// make the currently-ordered pane
	double total = 0;
	list[0] = "";
	for(ndx = 1, i = 0; i < nextorder; i++, ndx++) {
		snprintf(temp, sizeof(temp), CURRENTLY_ORDERED_QTY_COLOR("%2d") "   %20s " CURRENTLY_ORDERED_CURRENCY_COLOR("$%5.2f"), qty[i], orders[i], prices[i]);
		list[ndx] = copyChar(temp);
		//freeChar(orders[i]);

		if(specials_string[i] != NULL && strlen(specials_string[i]) > 0) {
			snprintf(temp, sizeof(temp), "     " CURRENTLY_ORDERED_SPECIAL_COLOR("%s"), specials_string[i]);
			list[++ndx] = copyChar(temp);
			//freeChar(specials_string[i]);
		}

		total += prices[i] * qty[i];
	}

	strcpy(temp, "");
	list[ndx++] = copyChar(temp);

	snprintf(temp, sizeof(temp), SUBTOTAL_COLOR("$%5.2f") " Subtotal", total);
	list[ndx++] = copyChar(temp);

	snprintf(temp, sizeof(temp), TAX_COLOR("$%5.2f") " Tax     ", TAX_RATE * total);
	list[ndx++] = copyChar(temp);

	snprintf(temp, sizeof(temp), TOTAL_COLOR("$%5.2f") " Total   ", total + (TAX_RATE * total));
	list[ndx++] = copyChar(temp);

	snprintf(temp, sizeof(temp), UNDERLINE("You have ordered the following:"));

	leftwindow = newCDKScroll(cdkscreen, 0, 4, NONE, LEFTWINDOW_HEIGHT, LEFTWINDOW_WIDTH, temp, list, ndx, FALSE, A_NORMAL, FALSE, FALSE);
	setCDKScrollBackgroundColor(leftwindow, ORDER_WINDOW_BACKGROUND_COLOR(" "));

	for(i = 1; i < ndx; i++)
		freeChar(list[i]);

	db_connect();
	char sql[500];
	PGresult *result;

	// make the customer details pane
	if(*customer_id == -1) {
		snprintf(temp, sizeof(temp), "Name: %s", "Unknown Customer");
		list[0] = copyChar(temp);
		snprintf(temp, sizeof(temp), "Tel: %s", remembered_callerid_phone_number);
		list[1] = copyChar(temp);
		snprintf(temp, sizeof(temp), "Address: %s", "Unknown Address");
		list[2] = copyChar(temp);
		snprintf(temp, sizeof(temp), "         ");
		list[3] = copyChar(temp);
		snprintf(temp, sizeof(temp), "Comment: ");
		list[4] = copyChar(temp);
		list[5] = copyChar(" ");
	} else {
		snprintf(sql, sizeof(sql), "SELECT name, phone, street, city, state, zip, other FROM customer WHERE customer_id='%d'", *customer_id);
		result = db_exec(sql);

		snprintf(temp, sizeof(temp), "Name: %s", db_get_value(result, 0, 0));
		list[0] = copyChar(temp);
		snprintf(temp, sizeof(temp), "Tel: %s", db_get_value(result, 0, 1));
		list[1] = copyChar(temp);
		snprintf(temp, sizeof(temp), "Address: %s", db_get_value(result, 0, 2));
		list[2] = copyChar(temp);
		snprintf(temp, sizeof(temp), "         %s, %s  %s", db_get_value(result, 0, 3), db_get_value(result, 0, 4), db_get_value(result, 0, 5));
		list[3] = copyChar(temp);
		snprintf(temp, sizeof(temp), "Comment: %s", db_get_value(result, 0, 6));
		list[4] = copyChar(temp);
		list[5] = copyChar(" ");

		db_clear(result);
	}

	if(current_pickup_or_delivery == ' ')
		snprintf(temp, sizeof(temp), BOLD("Eat In"));
	else if(current_pickup_or_delivery == 'P')
		snprintf(temp, sizeof(temp), BOLD("Carry Out"));
	else if(current_pickup_or_delivery == 'D')
		snprintf(temp, sizeof(temp), BOLD("Delivery"));
	list[6] = copyChar(temp);

	snprintf(temp, sizeof(temp), UNDERLINE("Customer Information:"));

	rightwindow = newCDKScroll(cdkscreen, LEFTWINDOW_WIDTH, 4, NONE, RIGHTWINDOW_HEIGHT, RIGHTWINDOW_WIDTH, temp, list, 7, FALSE, A_NORMAL, FALSE, FALSE);
	setCDKScrollBackgroundColor(rightwindow, CUSTOMERINFO_WINDOW_BACKGROUND_COLOR(" "));

	for(i = 0; i < 7; i++)
		freeChar(list[i]);

	// make the searchable menu pane
	snprintf(sql, sizeof(sql)-1, "SELECT item_id, name_eng, name_chi, price FROM Item ORDER BY item_id");
	result = db_exec(sql);

	num_rows = db_numrows(result);
	for(i = 0; i < num_rows; i++)
	{
		snprintf(temp, sizeof(temp)-1, "%s    %s    %s    $%0.2f",
				pad(db_get_value(result, i, 0), 5),
				pad(db_get_value(result, i, 1), 15),
				pad(db_get_value(result, i, 2), 15),
				atof(db_get_value(result, i, 3))
				);
		list[i] = copyChar(temp);
	}

	db_clear(result);

	strcpy(temp, "");
	snprintf(temp2, sizeof(temp2)-1, "</B>Quick Search: ");

	/* Create the alpha list. */
	nameList = newCDKAlphalist(cdkscreen, LEFTWINDOW_WIDTH - 1, RIGHTWINDOW_HEIGHT + 3, LEFTWINDOW_HEIGHT - RIGHTWINDOW_HEIGHT + 1, RIGHTWINDOW_WIDTH + 2,
					temp, temp2, list, num_rows,
					'_', A_REVERSE, FALSE, FALSE);
	setCDKAlphalistBackgroundColor(nameList, MENU_WINDOW_BACKGROUND_COLOR(" "));

	//refreshCDKScreen(cdkscreen);
	drawCDKAlphalist(nameList, nameList->box);
	drawCDKScroll(leftwindow, leftwindow->box);
	drawCDKScroll(rightwindow, rightwindow->box);

	setCDKAlphalistPreProcess(nameList, cb_orderScreen, nameList);

	char item_id[10];
	char name_eng[100];
	char name_chi[100];
	double price;

	char *selection = NULL;
	current_size = ' ';
	current_qty = 1;
	// Get the information they want to view.      
	selection = activateCDKAlphalist(nameList, (chtype *)NULL);
	if(nameList->exitType == vNORMAL)
	{
		// figure out which item we selected
		for(i = 0; i < num_rows; i++) {
			if(strstr(list[i], selection) == list[i]) {
				//fprintf(stderr, "Selection is '%s', you picked row %d.\n", selection, i);

				// build the order-line-item string
				split_item(list[i], item_id, name_eng, name_chi, &price);
				snprintf(temp, sizeof(temp), CURRENTLY_ORDERED_SIZE_COLOR("%c") " " CURRENTLY_ORDERED_ITEM_NAME_COLOR("%s %s %s"), current_size, pad(item_id, 5), pad(name_eng, 10), pad(name_chi, 10));

				int j;
				strcpy(temp2, "");
				for(j = 0; j < nextspecial; j++) {
					strncat(temp2, specials[j], sizeof(temp2));
					strncat(temp2, ", ", sizeof(temp2) - strlen(temp2));
					freeChar(specials[j]);
					price += specialprices[j];
				}
				if(temp2[strlen(temp2) - 1] == ' ' && temp2[strlen(temp2) - 2] == ',')
					temp2[strlen(temp2) - 2] = '\0';
				nextspecial = 0;

				if(lookup_order_index(item_id) == -1) {
					orders[nextorder] = copyChar(temp);
					specials_string[nextorder] = copyChar(temp2);
					qty[nextorder] += current_qty;
					prices[nextorder] = price;
					nextorder++;
				} else {
					for(j = 0; j < nextorder; j++) {
						if(strcmp(orders[j], temp) == 0 && strcmp(specials_string[j], temp2) == 0) {
							qty[j] += current_qty;
							break;
						}
					}

					if(j == nextorder) {
						orders[nextorder] = copyChar(temp);
						specials_string[nextorder] = copyChar(temp2);
						qty[nextorder] += current_qty;
						prices[nextorder] = price;
						nextorder++;
					}
				}

				//selection = NULL;
				//break;

				/* Clean up. */
				for(i = 0; i < num_rows; i++)
				{
					freeChar(list[i]);
				}
				eraseCDKAlphalist(nameList);
				destroyCDKAlphalist(nameList);
				destroyCDKScroll(leftwindow);
				destroyCDKScroll(rightwindow);
				db_disconnect();
				return 1;
			}
		}
	}
	// if escape
	if(nameList->exitType == vESCAPE_HIT)
	{
		if(nextorder > 0)
			ret = confirm("Complete the order?");
		else if(customer_id_has_changed == true)
			ret = confirm("Save this customer?");
		else
			ret = 1;

		if(ret == 0 && nextorder > 0) {
			int order_id;

			snprintf(sql, sizeof(sql)-1, "INSERT INTO orders (customer_id, order_date) VALUES ('%d', NOW())", *customer_id);
			db_exec(sql);

			snprintf(sql, sizeof(sql)-1, "SELECT last_value FROM orders_seq");
			result = db_exec(sql);
			order_id = atoi(db_get_value(result, 0, 0));
			db_clear(result);

			char *ptr;
			char item_id_str[100];
			for(i = 0; i < nextorder; i++) {
				// steal item_id from orders[i]
				ptr = strstr(orders[i], " ");
				strncpy(item_id_str, orders[i], ptr - orders[i]);
				item_id_str[ptr - orders[i]] = '\0';

				snprintf(sql, sizeof(sql)-1, "INSERT INTO orders_items (order_id, item_id) VALUES ('%d', '%s')", order_id, item_id_str);
				db_exec(sql);
			}
		} else if(ret == 1 && customer_id_has_changed == true) {
			// they don't want to save the customer, and they didn't order anything, so pwn them from the DB
			snprintf(sql, sizeof(sql), "DELETE FROM customer WHERE customer_id='%d'", *customer_id);
			db_exec(sql);
		}

		/* Clean up. */
		nextorder = 0;
		for(i = 0; i < num_rows; i++)
		{
			freeChar(list[i]);
		}
		eraseCDKAlphalist(nameList);
		destroyCDKAlphalist(nameList);
		destroyCDKScroll(leftwindow);
		destroyCDKScroll(rightwindow);
		db_disconnect();

		return 0;
	}

	// should never reach here, unless something truly horrible happens
	return 0;
}

int notImplemented()
{
	char *buttons[5], *mesg[15];
	mesg[0] = "<C>NOT IMPLEMENTED";
	mesg[1] = "<C>This feature is currently not implemented.";
	buttons[0] = "</B/24><OK>";
	return popupDialog(cdkscreen, mesg, 2, buttons, 1);
}

/*
 * This gets a phone record with all of the details.
 */
int getLargePhoneRecord(bool edit_flag)
{
	/* Declare local variables. */
	CDKENTRY *nameEntry, *addressEntry, *cityEntry;
	CDKENTRY *stateEntry, *postalEntry, *descEntry;
	CDKTEMPLATE *phoneTemplate;
	char *buttons[5], *mesg[15];
	int ret;

	/* Define the widgets. */
	nameEntry = newCDKEntry(cdkscreen, LEFT, 5,
							NULL, "</B/5>Name: ",
							A_NORMAL,
							'_', vMIXED, 20, 2, 256,
							TRUE, FALSE);
	addressEntry = newCDKEntry(cdkscreen, RIGHT, 5,
							NULL, "</B/5>Address: ",
							A_NORMAL,
							'_', vMIXED, 40, 2, 256,
							TRUE, FALSE);
	cityEntry = newCDKEntry(cdkscreen, LEFT, 8,
							NULL, "</B/5>City: ",
							A_NORMAL,
							'_', vMIXED, 20, 2, 256,
							TRUE, FALSE);
	stateEntry = newCDKEntry(cdkscreen, 29, 8,
							NULL, "</B/5>State: ",
							A_NORMAL,
							'_', vUCHAR, 15, 2, 256,
							TRUE, FALSE);
	postalEntry = newCDKEntry(cdkscreen, RIGHT, 8,
							NULL, "</B/5>Postal Code: ",
							A_NORMAL,
							'_', vUMIXED, 8, 2, 256,
							TRUE, FALSE);
	phoneTemplate = newCDKTemplate(cdkscreen, LEFT, 11,
							NULL,
							"</B/5>Number: ",
							"(###) ###-####",
							"(___) ___-____",
							TRUE, FALSE);
	descEntry = newCDKEntry(cdkscreen, RIGHT, 11,
							NULL,
							"</B/5>Description: ",
							A_NORMAL,
							'_', vMIXED, 20, 2, 256,
							TRUE, FALSE);

	/* Get the phone information. */
	for(;;)
	{
		/* Draw the widgets on the screen. */
		drawCDKEntry(nameEntry, nameEntry->box);
		drawCDKEntry(addressEntry, addressEntry->box);
		drawCDKEntry(cityEntry, cityEntry->box);
		drawCDKEntry(stateEntry, stateEntry->box);
		drawCDKEntry(postalEntry, postalEntry->box);
		//drawCDKTemplate(phoneTemplate, phoneTemplate->box);
		drawCDKEntry(descEntry, descEntry->box);

		/* Activate the entries to get the information. */

		char *name;
		char *address;
		char *city;
		char *state;
		char *postalCode;
		//char *phoneNumber;
		char *desc;

		name = copyChar(activateCDKEntry(nameEntry, NULL));
		address = copyChar(activateCDKEntry(addressEntry, NULL));
		city  = copyChar(activateCDKEntry(cityEntry, NULL));
		state = copyChar(activateCDKEntry(stateEntry, NULL));
		postalCode = copyChar(activateCDKEntry(postalEntry, NULL));
		//activateCDKTemplate(phoneTemplate, NULL);
		//phoneNumber = mixCDKTemplate(phoneTemplate);
		desc = copyChar(activateCDKEntry(descEntry, NULL));

		/* Determine if the user wants to submit the info. */
		mesg[0] = "<C></U>Confirm Entry";
		mesg[1] = "<C>Do you want to add this information?";

		if(edit_flag)
			buttons[0] = "</B/24><Update Menu Item>";
		else
			buttons[0] = "</B/24><Add Menu Item>";

		buttons[1] = "</B/16><Cancel>";
		buttons[2] = "</B/32><Modify Information>";
		ret = popupDialog(cdkscreen, mesg, 2, buttons, 3);

		/* Check the response of the popup dialog box. */
		if(ret == 0)
		{
			/* The user wants to submit the information. */
			destroyCDKEntry(nameEntry);
			destroyCDKEntry(addressEntry);
			destroyCDKEntry(cityEntry);
			destroyCDKEntry(stateEntry);
			destroyCDKEntry(postalEntry);
			destroyCDKEntry(descEntry);
			//destroyCDKTemplate(phoneTemplate);
			return ret;
		}
		else if(ret == 1)
		{
			/* The user does not want to submit the information. */
			freeChar(name);
			freeChar(address);
			freeChar(city);
			freeChar(state);
			freeChar(postalCode);
			//freeChar(phoneNumber);
			freeChar(desc);

			destroyCDKEntry(nameEntry);
			destroyCDKEntry(addressEntry);
			destroyCDKEntry(cityEntry);
			destroyCDKEntry(stateEntry);
			destroyCDKEntry(postalEntry);
			destroyCDKEntry(descEntry);
			//destroyCDKTemplate(phoneTemplate);
			return ret;
		}
		else
		{
			/* The user wants to edit the information again. */
			freeChar(name);
			freeChar(address);
			freeChar(city);
			freeChar(state);
			freeChar(postalCode);
			//freeChar(phoneNumber);
			freeChar(desc);
		}
	}
}

void split_item(char *string, char *item_id, char *name_eng, char *name_chi, double *price)
{
	char *ptr, *ptr2;

	ptr = strstr(string, "    ");
	strncpy(item_id, string, ptr - string);
	item_id[ptr - string] = '\0';

	while(*ptr == ' ') ptr++;
	ptr2 = strstr(ptr, "    ");
	strncpy(name_eng, ptr, ptr2 - ptr);
	name_eng[ptr2 - ptr] = '\0';

	while(*ptr2 == ' ') ptr2++;
	ptr = strstr(ptr2, "    ");
	strncpy(name_chi, ptr2, ptr - ptr2);
	name_chi[ptr - ptr2] = '\0';

	while(*ptr == ' ' || *ptr == '$') ptr++;
	*price = atof(ptr);
}

int lookup_order_index(char *string)
{
	int i;
	char *ptr;

	/* This function sucks, due to CDK's colorizing markup notation.
	 * The idea is to look through the orders[] array, and try to find
	 * a line where the item_id matches the 'string' argument; if we
	 * find a match, we return the index into the orders[] array; if we
	 * fail to match it, we return -1.  The problem is that the item_id
	 * is not the first thing on the line (the size comes first), and
	 * it is possible for the size and/or the item_id to be surrounded
	 * by CDK colorizing tags, which are not necessarily a fixed length.
	 * So, we cheat a bit.  If we notice any colorizing tags present,
	 * and we have a possible match on the item_id, we look for any more
	 * opening CDK colorizing tags to the right of the possible match;
	 * if we find any, we assume that it is not a match, and keep looking.
	 */

	for(i = 0; i < nextorder; i++) {
		ptr = strstr(orders[i], string);
		if(ptr != NULL) {
			if(strstr(orders[i], "<!") != NULL && strstr(orders[i], ">") != NULL) {
				if(*(ptr - 1) == '>') {
					while(*ptr != '\0') {
						if(*ptr == '<' && *(ptr + 1) == '/')
							break;
						ptr++;
					}
					if(*ptr == '\0')
						return i;
				}
			} else if(ptr == orders[i] + 2) {
				return i;
			}
		}
	}

	return -1;
}

int login()
{
	char username[100], password[100], sql[500], *ptr;
	int user_id;
	CDKENTRY *username_entry, *password_entry;

	username_entry = newCDKEntry(cdkscreen, LEFT, 5,
							NULL, "</B/5>Username: ",
							A_NORMAL,
							'_', vMIXED, 20, 2, 256,
							TRUE, FALSE);
	password_entry = newCDKEntry(cdkscreen, LEFT, 8,
							NULL, "</B/5>Password: ",
							A_NORMAL,
							'_', vHMIXED, 20, 2, 256,
							TRUE, FALSE);

	drawCDKEntry(username_entry, username_entry->box);
	drawCDKEntry(password_entry, password_entry->box);

	ptr = activateCDKEntry(username_entry, NULL);
	if(ptr == NULL){ return -1; }
	strncpy(username, ptr, sizeof(username));
	addslashes(username);

	ptr = activateCDKEntry(password_entry, NULL);
	if(ptr == NULL){ return -1; }
	strncpy(password, ptr, sizeof(password));
	addslashes(password);

	destroyCDKEntry(username_entry);
	destroyCDKEntry(password_entry);

	db_connect();
	snprintf(sql, sizeof(sql)-1, "SELECT user_id, name, employee_num FROM users WHERE username='%s' AND password='%s'", username, password);
	PGresult *result = db_exec(sql);

	if(db_numrows(result) > 0)
		user_id = atoi(db_get_value(result, 0, 0));
	else
		user_id = -1;

	db_clear(result);
	db_disconnect();
	return user_id;
}

int cb_orderScreen(EObjectType cdktype, void *object, void *data, chtype key)
{
	CDKALPHALIST *alphalist = (CDKALPHALIST*) data;
	CDKENTRY *widget = (CDKENTRY*) object;

	update_date_time();

	if(key == 10 || key == KEY_ENTER) {
		return 1;
	} else if(key == ADMIN_INTERFACE_KEY) {
		create_menu();
		menu();
		destroyCDKMenu(rolodexMenu);
		refreshCDKScreen(cdkscreen);
	} else if(key == PICKUP_KEY) {
		current_pickup_or_delivery = 'P';
	} else if(key == DELIVERY_KEY) {
		current_pickup_or_delivery = 'D';
	} else if(key == QUANTITY_KEY) {
		CDKENTRY *qty_entry = (CDKENTRY *)NULL;
		CDKLABEL *size_label = (CDKLABEL *)NULL;
		char *title[1];
		char sql[500];
		char temp[500], item_id[100], name_eng[100], name_chi[100];
		char *ptr;
		double price;
		int index, i;

		// figure out which menu item is highlighted
		snprintf(temp, sizeof(temp), "%s", getCDKEntryValue(widget));
		for(index = 0, i = 0; i < alphalist->listSize; i++) {
			ptr = strstr((alphalist->list)[i], temp);
			if(ptr == (alphalist->list)[i]) {
				index = i;
				break;
			}
		}

		// splice the menu item
		split_item((alphalist->list)[index], item_id, name_eng, name_chi, &price);

		db_connect();
		go_directly_to_specials = false;
		snprintf(sql, sizeof(sql)-1, "SELECT has_two_sizes FROM item WHERE item_id='%s'", item_id);
		PGresult *result = db_exec(sql);
		if(strcmp(db_get_value(result, 0, 0),"t") == 0) {
			title[0] = "</B>Size:<!B>     Pint ";
			size_label = newCDKLabel(cdkscreen, CENTER, 19, title, 1, FALSE, FALSE);
			drawCDKLabel(size_label, FALSE);
			qty_entry = newCDKEntry(cdkscreen, CENTER, 21,
									NULL, "</B>Quantity: ",
									A_NORMAL,
									'_', vINT, 5, 0, 5,
									FALSE, FALSE);

			setCDKEntryPreProcess(qty_entry, cb_size_qty_popup, size_label);

			// Get the information they want to view.      
			ptr = activateCDKEntry(qty_entry, NULL);

			if(ptr == NULL){ return -1; }
			if(strcmp(ptr, "") == 0)
				strcpy(ptr, "1");

			strncpy(temp, ptr, sizeof(temp));
			current_qty = atoi(temp);

			destroyCDKEntry(qty_entry);
			destroyCDKLabel(size_label);
		} else {
			int ch = 0;
			char buffer[20] = { '\0' };
			while(ch != 10) { // enter
				ch = getch();
				if(ch == 127) { // backspace
					buffer[strlen(buffer) - 1] = '\0';
				} else if(ch == SPECIALS_KEY) {
					go_directly_to_specials = true;
					break;
				} else if(ch >= '0' && ch <= '9') {
					buffer[strlen(buffer)] = ch;
				}
			}
			current_qty = atoi(buffer);
		}

		db_clear(result);

		if(go_directly_to_specials == true) {
			// call ourselves recursively to immediately process the SPECIALS_KEY
			cb_orderScreen(cdktype, object, data, SPECIALS_KEY);
		} else {
			ungetch(10); // throw an ENTER on the input buffer
		}

		refreshCDKScreen(cdkscreen);
	} else if(key == SPECIALS_KEY) {
		CDKSCROLL *nameList = (CDKSCROLL *)NULL;
		char *list[MAX_ITEMS], *ptr;
		char temp[500], item_id[100], name_eng[100], name_chi[100];
		double price;
		int index, i, j;

		// figure out which menu item is highlighted
		snprintf(temp, sizeof(temp), "%s", getCDKEntryValue(widget));
		for(index = 0, i = 0; i < alphalist->listSize; i++) {
			ptr = strstr((alphalist->list)[i], temp);
			if(ptr == (alphalist->list)[i]) {
				index = i;
				break;
			}
		}

		// splice the menu item
		split_item((alphalist->list)[index], item_id, name_eng, name_chi, &price);

		// loop through and let the user select as many specials as they want
		bool need_continue;
		while(true) {
			need_continue = false;

			db_connect();
			char sql[500];
			snprintf(sql, sizeof(sql)-1, "SELECT s.special_field_id, s.name_eng, s.name_chi, s.price FROM special_field as s, item_special_field as i WHERE i.special_field_id=s.special_field_id and i.item_id='%s' ORDER BY special_field_id", item_id);
			PGresult *result = db_exec(sql);

			int num_rows = db_numrows(result);

			// if there aren't any specials for the selected item, ignore the keystroke
			if(num_rows < 1) {
				return 0;
			}

			for(i = 0; i < num_rows; i++)
			{
				snprintf(temp, sizeof(temp)-1, "</B/29>%s  %s  $%0.2f",
						pad(db_get_value(result, i, 1), 15),
						pad(db_get_value(result, i, 2), 15),
						atof(db_get_value(result, i, 3))
						);
				list[i] = copyChar(temp);
			}

			snprintf(temp, sizeof(temp)-1, "<C></24></B>Special Fields in Database\n<C>Choose Special Fields, or ESCAPE to go back.\n");

			/* Create the scrolling list. */
			nameList = newCDKScroll(cdkscreen, CENTER, CENTER, RIGHT,
									-6, -4, temp, list, num_rows,
									NUMBERS, A_REVERSE, TRUE, FALSE);

			/* Clean up. */
			for(i = 0; i < num_rows; i++)
			{
				freeChar(list[i]);
			}

			setCDKScrollPreProcess(nameList, cb_specials_list, NULL);

			int selection = 0;
			// Get the information they want to view.      
			selection = activateCDKScroll(nameList, (chtype *)NULL);
			if(nameList->exitType == vNORMAL) {
				// if they select a special they've already selected, unselect it
				for(i = 0; i < nextspecial; i++) {
					if(strcmp(specials[i], db_get_value(result, selection, 1)) == 0) {
						freeChar(specials[i]);
						for(j = i + 1; j < nextspecial; j++) {
							specials[j - 1] = copyChar(specials[j]);
							specialprices[j - 1] = specialprices[j];
							freeChar(specials[j]);
						}
						nextspecial--;
						need_continue = true;
						break;
					}
				}

				if(need_continue == true)
					continue;

				if(i == nextspecial) {
					specials[nextspecial] = copyChar(db_get_value(result, selection, 1));
					specialprices[nextspecial] = atof(db_get_value(result, selection, 3));
					nextspecial++;
				}

				eraseCDKScroll(nameList);
				destroyCDKScroll(nameList);
				db_clear(result);
				//db_disconnect();
				refreshCDKScreen(cdkscreen);
			} else if(nameList->exitType == vESCAPE_HIT) {
				//nextspecial = 0;

				eraseCDKScroll(nameList);
				destroyCDKScroll(nameList);
				db_clear(result);
				//db_disconnect();
				refreshCDKScreen(cdkscreen);
				break;
			}
		}

		ungetch(10); // throw an ENTER on the input buffer

	} else if(key == EDIT_CUSTOMER_INFO_KEY) {
		int ch = 0;
		char buffer[100];
		char c_name[100], c_address[100], c_city[100], c_state[3], c_zip[6], c_phone[11], c_other[100];
		bool save_and_quit = false;
		char sql[500];

		// Get current info from DB
		db_connect();
		snprintf(sql, sizeof(sql)-1, "SELECT name, street, city, state, zip, phone, number_calls, other FROM Customer WHERE customer_id='%d'", customer_id);
		PGresult *result = db_exec(sql);

		strcpy(c_name, db_get_value(result, 0, 0));
		strcpy(c_address, db_get_value(result, 0, 1));
		strcpy(c_city, db_get_value(result, 0, 2));
		strcpy(c_state, db_get_value(result, 0, 3));
		strcpy(c_zip, db_get_value(result, 0, 4));
		strcpy(c_phone, db_get_value(result, 0, 5));
		strcpy(c_other, db_get_value(result, 0, 7));

		db_clear(result);

		// Customer Name field
		memset(buffer, '\0', sizeof(buffer));
		strcpy(buffer, c_name);
		mvaddstr(4 + 2, LEFTWINDOW_WIDTH + 7, buffer);
		move(4 + 2, LEFTWINDOW_WIDTH + 7 + strlen(buffer));
		refresh();
		while(ch != 10) { // enter
			ch = getch();
			if(ch == 10) {
				break;
			} else if(ch == KEY_BACKSPACE) {
				buffer[strlen(buffer) - 1] = '\0';
				mvaddstr(4 + 2, LEFTWINDOW_WIDTH + 7, buffer);
				addch(' ');
				move(4 + 2, LEFTWINDOW_WIDTH + 7 + strlen(buffer));
			} else if(ch == STOP_EDIT_CUSTOMER_INFO_KEY) {
				save_and_quit = true;
				break;
			} else {
				addch(ch);
				buffer[strlen(buffer)] = ch;
			}
		}
		ch = 0;
		strcpy(c_name, buffer);

		if(customer_id == 0) {
			snprintf(sql, sizeof(sql)-1, "INSERT INTO customer (name, street, city, state, zip, phone, other, number_calls) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '')", c_name, c_address, c_city, c_state, c_zip, c_phone, c_other);
			db_exec(sql);
			snprintf(sql, sizeof(sql)-1, "SELECT last_value FROM customer_seq");
			result = db_exec(sql);

			customer_id = atoi(db_get_value(result, 0, 0));
			customer_id_has_changed = true;

			db_clear(result);
		} else {
			snprintf(sql, sizeof(sql)-1, "UPDATE customer SET name='%s' WHERE customer_id='%d'", c_name, customer_id);
			db_exec(sql);
		}

		mvaddstr(4 + 2, LEFTWINDOW_WIDTH + 7, "                       ");
		refresh();

		update_rightwindow();

		if(save_and_quit) {
			//db_disconnect();
			move(RIGHTWINDOW_HEIGHT + 4 + 1, LEFTWINDOW_WIDTH + 15);
			refresh();
			return 0;
		}

		// Customer Phone field
		memset(buffer, '\0', sizeof(buffer));
		strcpy(buffer, c_phone);
		mvaddstr(4 + 3, LEFTWINDOW_WIDTH + 6, buffer);
		move(4 + 3, LEFTWINDOW_WIDTH + 6 + strlen(buffer));
		refresh();
		while(ch != 10) { // enter
			ch = getch();
			if(ch == 10) {
				break;
			} else if(ch == KEY_BACKSPACE) {
				buffer[strlen(buffer) - 1] = '\0';
				mvaddstr(4 + 3, LEFTWINDOW_WIDTH + 6, buffer);
				addch(' ');
				move(4 + 3, LEFTWINDOW_WIDTH + 6 + strlen(buffer));
			} else if(ch == STOP_EDIT_CUSTOMER_INFO_KEY) {
				save_and_quit = true;
				break;
			} else if(strlen(buffer) < 10 && ch >= '0' && ch <= '9') {
				addch(ch);
				buffer[strlen(buffer)] = ch;
			}
		}
		ch = 0;
		strcpy(c_phone, buffer);

		snprintf(sql, sizeof(sql)-1, "UPDATE customer SET phone='%s' WHERE customer_id='%d'", c_phone, customer_id);
		db_exec(sql);

		mvaddstr(4 + 3, LEFTWINDOW_WIDTH + 6, "                       ");
		refresh();
		update_rightwindow();

		if(save_and_quit) {
			//db_disconnect();
			move(RIGHTWINDOW_HEIGHT + 4 + 1, LEFTWINDOW_WIDTH + 15);
			refresh();
			return 0;
		}

		// Customer Address field
		memset(buffer, '\0', sizeof(buffer));
		strcpy(buffer, c_address);
		mvaddstr(4 + 4, LEFTWINDOW_WIDTH + 10, buffer);
		move(4 + 4, LEFTWINDOW_WIDTH + 10 + strlen(buffer));
		refresh();
		while(ch != 10) { // enter
			ch = getch();
			if(ch == 10) {
				break;
			} else if(ch == KEY_BACKSPACE) {
				buffer[strlen(buffer) - 1] = '\0';
				mvaddstr(4 + 4, LEFTWINDOW_WIDTH + 10, buffer);
				addch(' ');
				move(4 + 4, LEFTWINDOW_WIDTH + 10 + strlen(buffer));
			} else if(ch == STOP_EDIT_CUSTOMER_INFO_KEY) {
				save_and_quit = true;
				break;
			} else {
				addch(ch);
				buffer[strlen(buffer)] = ch;
			}
		}
		ch = 0;
		strcpy(c_address, buffer);

		snprintf(sql, sizeof(sql)-1, "UPDATE customer SET street='%s' WHERE customer_id='%d'", c_address, customer_id);
		db_exec(sql);

		mvaddstr(4 + 4, LEFTWINDOW_WIDTH + 10, "                       ");
		refresh();
		update_rightwindow();

		if(save_and_quit) {
			//db_disconnect();
			move(RIGHTWINDOW_HEIGHT + 4 + 1, LEFTWINDOW_WIDTH + 15);
			refresh();
			return 0;
		}

		// Customer City field
		memset(buffer, '\0', sizeof(buffer));
		strcpy(buffer, c_city);
		mvaddstr(4 + 5, LEFTWINDOW_WIDTH + 10, buffer);
		move(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(buffer));
		refresh();
		while(ch != 10) { // enter
			ch = getch();
			if(ch == 10) {
				break;
			} else if(ch == KEY_BACKSPACE) {
				buffer[strlen(buffer) - 1] = '\0';
				mvaddstr(4 + 5, LEFTWINDOW_WIDTH + 10, buffer);
				addch(' ');
				move(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(buffer));
			} else if(ch == STOP_EDIT_CUSTOMER_INFO_KEY) {
				save_and_quit = true;
				break;
			} else {
				addch(ch);
				buffer[strlen(buffer)] = ch;
			}
		}
		ch = 0;
		strcpy(c_city, buffer);

		snprintf(sql, sizeof(sql)-1, "UPDATE customer SET city='%s' WHERE customer_id='%d'", c_city, customer_id);
		db_exec(sql);

		mvaddstr(4 + 5, LEFTWINDOW_WIDTH + 10, "                       ");
		refresh();
		update_rightwindow();

		if(save_and_quit) {
			//db_disconnect();
			move(RIGHTWINDOW_HEIGHT + 4 + 1, LEFTWINDOW_WIDTH + 15);
			refresh();
			return 0;
		}

		// Customer State field
		memset(buffer, '\0', sizeof(buffer));
		strcpy(buffer, c_state);
		mvaddstr(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(c_city) + 2, buffer);
		move(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(c_city) + 2 + strlen(buffer));
		refresh();
		while(ch != 10) { // enter
			ch = getch();
			if(ch == 10) {
				break;
			} else if(ch == KEY_BACKSPACE) {
				buffer[strlen(buffer) - 1] = '\0';
				mvaddstr(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(c_city) + 2, buffer);
				addch(' ');
				move(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(c_city) + 2 + strlen(buffer));
			} else if(ch == STOP_EDIT_CUSTOMER_INFO_KEY) {
				save_and_quit = true;
				break;
			} else if(strlen(buffer) < 2 && ((ch >= 'A' && ch <= 'Z') || (ch >= 'a' && ch <= 'z'))) {
				addch(toupper(ch));
				buffer[strlen(buffer)] = toupper(ch);
			}
		}
		ch = 0;
		strcpy(c_state, buffer);

		snprintf(sql, sizeof(sql)-1, "UPDATE customer SET state='%s' WHERE customer_id='%d'", c_state, customer_id);
		db_exec(sql);

		mvaddstr(4 + 5, LEFTWINDOW_WIDTH + 10, "                       ");
		refresh();
		update_rightwindow();

		if(save_and_quit) {
			//db_disconnect();
			move(RIGHTWINDOW_HEIGHT + 4 + 1, LEFTWINDOW_WIDTH + 15);
			refresh();
			return 0;
		}

		// Customer Zip field
		memset(buffer, '\0', sizeof(buffer));
		strcpy(buffer, c_zip);
		mvaddstr(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(c_city) + 2 + strlen(c_state) + 2, buffer);
		move(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(c_city) + 2 + strlen(c_state) + 2 + strlen(buffer));
		refresh();
		while(ch != 10) { // enter
			ch = getch();
			if(ch == 10) {
				break;
			} else if(ch == KEY_BACKSPACE) {
				buffer[strlen(buffer) - 1] = '\0';
				mvaddstr(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(c_city) + 2 + strlen(c_state) + 2, buffer);
				addch(' ');
				move(4 + 5, LEFTWINDOW_WIDTH + 10 + strlen(c_city) + 2 + strlen(c_state) + 2 + strlen(buffer));
			} else if(ch == STOP_EDIT_CUSTOMER_INFO_KEY) {
				save_and_quit = true;
				break;
			} else if(strlen(buffer) < 5 && ch >= '0' && ch <= '9') {
				addch(ch);
				buffer[strlen(buffer)] = ch;
			}
		}
		ch = 0;
		strcpy(c_zip, buffer);

		snprintf(sql, sizeof(sql)-1, "UPDATE customer SET zip='%s' WHERE customer_id='%d'", c_zip, customer_id);
		db_exec(sql);

		mvaddstr(4 + 5, LEFTWINDOW_WIDTH + 10, "                       ");
		refresh();
		update_rightwindow();

		if(save_and_quit) {
			//db_disconnect();
			move(RIGHTWINDOW_HEIGHT + 4 + 1, LEFTWINDOW_WIDTH + 15);
			refresh();
			return 0;
		}

		// Customer Other field
		memset(buffer, '\0', sizeof(buffer));
		strcpy(buffer, c_other);
		mvaddstr(4 + 6, LEFTWINDOW_WIDTH + 10, buffer);
		move(4 + 6, LEFTWINDOW_WIDTH + 10 + strlen(buffer));
		refresh();
		while(ch != 10) { // enter
			ch = getch();
			if(ch == 10) {
				break;
			} else if(ch == KEY_BACKSPACE) {
				buffer[strlen(buffer) - 1] = '\0';
				mvaddstr(4 + 6, LEFTWINDOW_WIDTH + 10, buffer);
				addch(' ');
				move(4 + 6, LEFTWINDOW_WIDTH + 10 + strlen(buffer));
			} else if(ch == STOP_EDIT_CUSTOMER_INFO_KEY) {
				save_and_quit = true;
				break;
			} else {
				addch(ch);
				buffer[strlen(buffer)] = ch;
			}
		}
		ch = 0;
		strcpy(c_other, buffer);

		snprintf(sql, sizeof(sql)-1, "UPDATE customer SET other='%s' WHERE customer_id='%d'", c_other, customer_id);
		db_exec(sql);

		mvaddstr(4 + 6, LEFTWINDOW_WIDTH + 10, "                       ");
		refresh();
		update_rightwindow();

		//db_disconnect();
		move(RIGHTWINDOW_HEIGHT + 4 + 1, LEFTWINDOW_WIDTH + 15);
		refresh();
		return 0;
	} else if(key == KEY_DOWN || key == KEY_UP || key == KEY_NPAGE || key == KEY_PPAGE) {
		// clear out the entry box, except for the item id (FIXME: this doesn't work)
		char *ptr, *ptr2;
		ptr = getCDKEntryValue(widget);
		ptr2 = strstr(ptr, " ");
		if(ptr2 != NULL) *ptr2 = '\0';
		setCDKEntryValue(widget, ptr);

		nextspecial = 0;
		return 1;
#ifdef ESCAPE_REPLACEMENT_KEY
	} else if(key == ESCAPE_REPLACEMENT_KEY) {
		ungetch(KEY_ESC);
		return 0;
#endif
	} else {
		// for every other normal character, we need to do the search & highlight trick ourself,
		// since we're overriding the alphalist's builtin preprocess function on the entry box.
		int index, i;
		char *ptr, temp[100];

		snprintf(temp, sizeof(temp), "%s%c", getCDKEntryValue(widget), (unsigned) key);
		for(index = 0, i = 0; i < alphalist->listSize; i++) {
			ptr = strstr((alphalist->list)[i], temp);
			if(ptr == (alphalist->list)[i]) {
				index = i;
				break;
			}
		}

		if(index < 0) {
			Beep();
			return 0;
		}

		nextspecial = 0;
		eraseCDKScroll(alphalist->scrollField);
		setCDKScrollPosition(alphalist->scrollField, index);
		drawCDKScroll(alphalist->scrollField, (alphalist->scrollField)->box);

		return 1;
	}

	return 0;
}

void *callerid_func()
{
	int fd, result;
	struct termios oldtio, newtio;
	char inbuf[256], outbuf[256];
	char *ptr;
	bool ready;

	// initialize the modem
	fd = open(MODEM_DEVICE, O_RDWR | O_NOCTTY);
	if(fd < 0) {
		fprintf(stderr, "Failed to open modem device; caller ID _NOT_ enabled.\n");
		return NULL;
	}

	// termios magic
	tcgetattr(fd, &oldtio);
	memset(&newtio, 0, sizeof(newtio));
	newtio.c_cflag = B9600 | CRTSCTS | CS8 | CLOCAL | CREAD;
	newtio.c_iflag = IGNPAR;
	newtio.c_oflag = 0;
	newtio.c_lflag = 0;
	newtio.c_cc[VTIME] = 20;
	newtio.c_cc[VMIN] = 0;
	tcflush(fd, TCIFLUSH);
	tcsetattr(fd, TCSANOW, &newtio);

	write(fd, MODEM_INIT_STRING, strlen(MODEM_INIT_STRING));

	memset(&outbuf, 0, sizeof(outbuf));
	while(true) {
		// if there's already a caller ID order awaiting processing, block.
		ready = false;
		while(!ready) {
			pthread_mutex_lock(&mutex);
			if(strcmp(callerid_phone_number, "") == 0)
				ready = true;
			pthread_mutex_unlock(&mutex);
		}

		result = read(fd, inbuf, sizeof(inbuf));
		inbuf[result] = '\0';
		strcat(outbuf, inbuf);

		if((ptr = strstr(outbuf, "NMBR = ")) != NULL && strlen(ptr) >= 7 + 10) {
			pthread_mutex_lock(&mutex);
			strcpy(callerid_phone_number, ptr + 7);
			callerid_phone_number[10] = '\0';
			pthread_mutex_unlock(&mutex);
			memset(&outbuf, 0, sizeof(outbuf));
		}
	}

	tcsetattr(fd, TCSANOW, &oldtio);
	return NULL;
}

int cb_size_qty_popup(EObjectType cdktype, void *object, void *data, chtype key)
{
	//CDKENTRY *widget = (CDKENTRY*) object;
	CDKLABEL *label = (CDKLABEL*) data;
	char *title[1];

	if(key == KEY_DOWN) {
		current_size = 'P';
		title[0] = "</B>Size:<!B>     Pint ";
		setCDKLabelMessage(label, title, 1);
		return 0;
	} else if(key == KEY_UP) {
		current_size = 'Q';
		title[0] = "</B>Size:<!B>     Quart";
		setCDKLabelMessage(label, title, 1);
		return 0;
	} else if(key == SPECIALS_KEY) {
		go_directly_to_specials = true;
		ungetch(10); // throw an ENTER on the input buffer
		return 0;
#ifdef ESCAPE_REPLACEMENT_KEY
	} else if(key == ESCAPE_REPLACEMENT_KEY) {
		ungetch(KEY_ESC);
		return 0;
#endif
	} else {
		return 1;
	}
}

int cb_specials_list(EObjectType cdktype, void *object, void *data, chtype key)
{
	//CDKENTRY *widget = (CDKENTRY*) object;

	if(key == SPECIALS_KEY) {
		ungetch(KEY_ESC);
		return 0;
#ifdef ESCAPE_REPLACEMENT_KEY
	} else if(key == ESCAPE_REPLACEMENT_KEY) {
		ungetch(KEY_ESC);
		return 0;
#endif
	} else {
		return 1;
	}
}

void update_rightwindow()
{
	char temp[500];
	char *list[MAX_ITEMS];
	char sql[500];
	PGresult *result;
	int i;

	// make the customer details pane
	db_connect();
	snprintf(sql, sizeof(sql), "SELECT name, phone, street, city, state, zip, other FROM customer WHERE customer_id='%d'", customer_id);
	result = db_exec(sql);

	snprintf(temp, sizeof(temp), "Name: %s", db_get_value(result, 0, 0));
	list[0] = copyChar(temp);
	snprintf(temp, sizeof(temp), "Tel: %s", db_get_value(result, 0, 1));
	list[1] = copyChar(temp);
	snprintf(temp, sizeof(temp), "Address: %s", db_get_value(result, 0, 2));
	list[2] = copyChar(temp);
	snprintf(temp, sizeof(temp), "         %s, %s  %s", db_get_value(result, 0, 3), db_get_value(result, 0, 4), db_get_value(result, 0, 5));
	list[3] = copyChar(temp);
	snprintf(temp, sizeof(temp), "Comment: %s", db_get_value(result, 0, 6));
	list[4] = copyChar(temp);
	list[5] = copyChar(" ");

	db_clear(result);

	if(current_pickup_or_delivery == ' ')
		snprintf(temp, sizeof(temp), BOLD("Eat In"));
	else if(current_pickup_or_delivery == 'P')
		snprintf(temp, sizeof(temp), BOLD("Carry Out"));
	else if(current_pickup_or_delivery == 'D')
		snprintf(temp, sizeof(temp), BOLD("Delivery"));
	list[6] = copyChar(temp);

	snprintf(temp, sizeof(temp), UNDERLINE("Customer Information:"));

	setCDKScrollItems(rightwindow, list, 7, FALSE);

	for(i = 0; i < 7; i++)
		freeChar(list[i]);

	drawCDKScroll(rightwindow, rightwindow->box);

	db_disconnect();
}
