/************************************************************************
 * Chinese Point-of-Sale system                                         *
 * Copyright (C) 2004  Cerebral Tech, Inc.                              *
 * Email: contact@cerebraltech.com                                      *
 * Web:   www.cerebraltech.com                                          *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 ***********************************************************************/

#ifndef __POS_H
#define __POS_H

#include <cdk.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <pthread.h>
#include <signal.h>
#include "pg_database.h"

// the name of this store
#ifndef STORE_NAME
#define STORE_NAME "Example Store"
#endif

// max number of unique items in an order (different sizes / specials are considered separate items)
#ifndef MAX_ORDER_SIZE
#define MAX_ORDER_SIZE 100
#endif

// tax rate (as a decimal, e.g. 5% = "0.05" (should be defined in Makefile; this is a default value)
#ifndef TAX_RATE
#define TAX_RATE 0.05
#endif

// key bindings (should be defined in Makefile; these are default values)
#ifndef DELIVERY_KEY
#define DELIVERY_KEY 'l'
#endif

#ifndef PICKUP_KEY
#define PICKUP_KEY 'p'
#endif

#ifndef SPECIALS_KEY
#define SPECIALS_KEY '/'
#endif

#ifndef QUANTITY_KEY
#define QUANTITY_KEY '*'
#endif

#ifndef EDIT_CUSTOMER_INFO_KEY
#define EDIT_CUSTOMER_INFO_KEY KEY_F12
#endif

#ifndef STOP_EDIT_CUSTOMER_INFO_KEY
#define STOP_EDIT_CUSTOMER_INFO_KEY KEY_F11
#endif

#ifndef ADMIN_INTERFACE_KEY
#define ADMIN_INTERFACE_KEY KEY_F2
#endif

// window dimensions (should be defined in Makefile; these are default values)
#ifndef LEFTWINDOW_HEIGHT
#define LEFTWINDOW_HEIGHT 23
#endif

#ifndef LEFTWINDOW_WIDTH
#define LEFTWINDOW_WIDTH 50
#endif

#ifndef RIGHTWINDOW_HEIGHT
#define RIGHTWINDOW_HEIGHT 11
#endif

#ifndef RIGHTWINDOW_WIDTH
#define RIGHTWINDOW_WIDTH 25
#endif

// modem device (should be defined in Makefile; this is a default value)
#ifndef MODEM_DEVICE
#define MODEM_DEVICE "/dev/modem"
#endif

// modem initialization string (auto-answer off, caller-id on)
#ifndef MODEM_INIT_STRING
#define MODEM_INIT_STRING "ATE1V1Q0 S0=0 +VCID=1\r\n"
#endif

// friendly macros for the CDK color codes
#define BOLD(x) "</B>" x
#define UNDERLINE(x) "</U>" x
#define WHITE_ON_WHITE(x) "</01>" x "<!01>"
#define WHITE_ON_RED(x) "</02>" x "<!02>"
#define WHITE_ON_GREEN(x) "</03>" x "<!03>"
#define WHITE_ON_YELLOW(x) "</04>" x "<!04>"
#define WHITE_ON_BLUE(x) "</05>" x "<!05>"
#define WHITE_ON_PURPLE(x) "</06>" x "<!06>"
#define WHITE_ON_CYAN(x) "</07>" x "<!07>"
#define WHITE_ON_BLACK(x) "</08>" x "<!08>"
#define RED_ON_WHITE(x) "</09>" x "<!09>"
#define RED_ON_RED(x) "</10>" x "<!10>"
#define RED_ON_GREEN(x) "</11>" x "<!11>"
#define RED_ON_YELLOW(x) "</12>" x "<!12>"
#define RED_ON_BLUE(x) "</13>" x "<!13>"
#define RED_ON_PURPLE(x) "</14>" x "<!14>"
#define RED_ON_CYAN(x) "</15>" x "<!15>"
#define RED_ON_BLACK(x) "</16>" x "<!16>"
#define GREEN_ON_WHITE(x) "</17>" x "<!17>"
#define GREEN_ON_RED(x) "</18>" x "<!18>"
#define GREEN_ON_GREEN(x) "</19>" x "<!19>"
#define GREEN_ON_YELLOW(x) "</20>" x "<!20>"
#define GREEN_ON_BLUE(x) "</21>" x "<!21>"
#define GREEN_ON_PURPLE(x) "</22>" x "<!22>"
#define GREEN_ON_CYAN(x) "</23>" x "<!23>"
#define GREEN_ON_BLACK(x) "</24>" x "<!24>"
#define YELLOW_ON_WHITE(x) "</25>" x "<!25>"
#define YELLOW_ON_RED(x) "</26>" x "<!26>"
#define YELLOW_ON_GREEN(x) "</27>" x "<!27>"
#define YELLOW_ON_YELLOW(x) "</28>" x "<!28>"
#define YELLOW_ON_BLUE(x) "</29>" x "<!29>"
#define YELLOW_ON_PURPLE(x) "</30>" x "<!30>"
#define YELLOW_ON_CYAN(x) "</31>" x "<!31>"
#define YELLOW_ON_BLACK(x) "</32>" x "<!32>"
#define BLUE_ON_WHITE(x) "</33>" x "<!33>"
#define BLUE_ON_RED(x) "</34>" x "<!34>"
#define BLUE_ON_GREEN(x) "</35>" x "<!35>"
#define BLUE_ON_YELLOW(x) "</36>" x "<!36>"
#define BLUE_ON_BLUE(x) "</37>" x "<!37>"
#define BLUE_ON_PURPLE(x) "</38>" x "<!38>"
#define BLUE_ON_CYAN(x) "</39>" x "<!39>"
#define BLUE_ON_BLACK(x) "</40>" x "<!40>"
#define PURPLE_ON_WHITE(x) "</41>" x "<!41>"
#define PURPLE_ON_RED(x) "</42>" x "<!42>"
#define PURPLE_ON_GREEN(x) "</43>" x "<!43>"
#define PURPLE_ON_YELLOW(x) "</44>" x "<!44>"
#define PURPLE_ON_BLUE(x) "</45>" x "<!45>"
#define PURPLE_ON_PURPLE(x) "</46>" x "<!46>"
#define PURPLE_ON_CYAN(x) "</47>" x "<!47>"
#define PURPLE_ON_BLACK(x) "</48>" x "<!48>"
#define CYAN_ON_WHITE(x) "</49>" x "<!49>"
#define CYAN_ON_RED(x) "</50>" x "<!50>"
#define CYAN_ON_GREEN(x) "</51>" x "<!51>"
#define CYAN_ON_YELLOW(x) "</52>" x "<!52>"
#define CYAN_ON_BLUE(x) "</53>" x "<!53>"
#define CYAN_ON_PURPLE(x) "</54>" x "<!54>"
#define CYAN_ON_CYAN(x) "</55>" x "<!55>"
#define CYAN_ON_BLACK(x) "</56>" x "<!56>"
#define BLACK_ON_WHITE(x) "</57>" x "<!57>"
#define BLACK_ON_RED(x) "</58>" x "<!58>"
#define BLACK_ON_GREEN(x) "</59>" x "<!59>"
#define BLACK_ON_YELLOW(x) "</60>" x "<!60>"
#define BLACK_ON_BLUE(x) "</61>" x "<!61>"
#define BLACK_ON_PURPLE(x) "</62>" x "<!62>"
#define BLACK_ON_CYAN(x) "</63>" x "<!63>"

// color choices
#define CUSTOMERINFO_WINDOW_BACKGROUND_COLOR	WHITE_ON_RED
#define MENU_WINDOW_BACKGROUND_COLOR			WHITE_ON_BLUE
#define ORDER_WINDOW_BACKGROUND_COLOR			WHITE_ON_GREEN
#define CURRENTLY_ORDERED_CURRENCY_COLOR		RED_ON_GREEN
#define CURRENTLY_ORDERED_QTY_COLOR				PURPLE_ON_GREEN
#define CURRENTLY_ORDERED_ITEM_NAME_COLOR		BLACK_ON_GREEN
#define CURRENTLY_ORDERED_SIZE_COLOR			WHITE_ON_GREEN
#define CURRENTLY_ORDERED_SPECIAL_COLOR			BLUE_ON_GREEN
#define SUBTOTAL_COLOR							WHITE_ON_GREEN
#define TAX_COLOR								UNDERLINE(WHITE_ON_GREEN)
#define TOTAL_COLOR								BOLD(WHITE_ON_GREEN)
#define TIME_COLOR								BOLD(WHITE_ON_BLACK)
#define DATE_COLOR								BOLD(WHITE_ON_BLACK)
#define STORE_NAME_COLOR						BOLD(WHITE_ON_BLACK)

// Function Prototypes
int editMenuItem(bool edit_flag, char *db_item_id);
int editSpecial(bool edit_flag, int db_special_id);
int editCustomer(bool edit_flag, int db_customer_id);
int confirm(char *mesg);
char *pad(char *string, int length);
char *addslashes(char *string);
int viewMenuItems();
int viewSpecials();
int viewCustomers(bool edit_flag);
int orderScreen(int *customer_id);
int notImplemented();
int getLargePhoneRecord(bool edit_flag);
void create_menu();
int menu();
int login();
void update_rightwindow();
void update_date_time();
void split_item(char *string, char *item_id, char *name_eng, char *name_chi, double *price);
int lookup_order_index(char *item_id);
int cb_orderScreen(EObjectType cdktype, void *object, void *data, chtype key);
int cb_size_qty_popup(EObjectType cdktype, void *object, void *data, chtype key);
int cb_specials_list(EObjectType cdktype, void *object, void *data, chtype key);
void *callerid_func();

#endif
